
//
// A subroutine that applies an operator to an input dataset
// and writes the result to stdout.
//

#include <SEPinit.h>

#include <floatspacea.h>
#include <foperator.h>
#include <fopadjoint.h>

#include <Process.h>

#include <hestsolver.h>


floatspacearray Process( const floatspacearray &input, 
			    foperator & op, 
			    int adj, int inv, int niter) {
	
	// construct the empty output
    floatspacearray output;

    if( inv == 0 ){  

      // just apply the operator forward or transpose

      if( adj == 0 ) {
	  output = (op.Forward(input));
      } else {
	  output = (op.Adjoint(input));
      }

    }else{

      // solve the inverse problem

      if( adj == 0 ){

	HestSolver solver( op );
	solver.SetMaxiter( niter );
	output = solver.Solve( input );   // solve with initial guees of zero

      }else{

        // make Adjoint operator
        fopAdjoint opa( op );
	
	HestSolver solver( opa );
	solver.SetMaxiter( niter );
	output = solver.Solve( input );   // solve with initial guess of zero

      }

    }

    return output;
      
}
