#include <hestsolver.h>

floatspacearray HestSolver::Solve( const floatspacearray & rhs,
				   const floatspacearray & init  ) {
    floatspacearray x,step,cstep,grad,resid;     // work space
    double gamma,gamma1,alpha,beta;

    rhs_norm = rhs.norm();                      // initial norm
    x = init; resid = rhs - op->Forward(x);     // initial residual

    grad = op->Adjoint(resid);
    step = grad;  gamma1 = step.norm2();

    numiter=0;
    do{ 
	cstep = op->Forward(step);
	alpha = gamma1 / cstep.norm2();
	x += step * alpha;
	cstep *= alpha; resid -= cstep;
	grad = op->Adjoint(resid);
	gamma = grad.norm2();
	beta = gamma / gamma1;
	gamma1 = gamma;
	step *= beta; step += grad;

	// gather statistics, report, save iterations etc.
        resid_norm = resid.norm();
	report(); save_one_iter( x ); save_one_resid( resid ); 

    }while( (numiter++ <maxiter) && ( resid_norm / rhs_norm > 1e-8 ) );

    return x;
}
