#ifndef ITERSOLVER_H
#define ITERSOLVER_H
// abstract base class for iterative least squares solver, 

#include <lssolver.h>
#include <iostream.h>

class IterSolver : public LSSolver {

public:

IterSolver(foperator & fop , int maxit=10 ) : 
	LSSolver( fop ), save_x_iter(0), save_resid(0), 
	maxiter(maxit), numiter(0) , repstream(nil){};

virtual floatspacearray Solve(const floatspacearray & rhs,
                              const floatspacearray& init) = 0; //pure virtual

floatspacearray Solve(  const floatspacearray & rhs );

void ReportOn( ostream & rep ) 			// report on this stream
 	{repstream = &rep; }  

void SetSaveResidStep( int save_step );		// set residual save interval
void SetSaveIterStep( int save_step );		// set iteration save interval

int GetIterCount(){ return numiter; }		// number of iterations used

floatspacearray GetResiduals( );		// retrieve all residuals
floatspacearray GetOneResid( int i );		// retrieve one residual

floatspacearray GetIterations();		// retrieve all iterations
floatspacearray GetOneIter( int i );		// retrieve one iteration

~IterSolver();

protected:
int maxiter; 		// maximum number of iterations to perform

int numiter; 		// number of iterations actually performed

virtual void report();  // report function, reports statistics
ostream * repstream;
double rhs_norm;        // initial rhs norm
double resid_norm;      // current residual norm

int save_x_iter;
floatspacearray **saved_x_iter;
void save_one_iter( const floatspacearray & x );

int save_resid;
floatspacearray **saved_resid;
void save_one_resid( const floatspacearray & x );

};
#endif
