#include <normcgsolver.h>

floatspacearray NormCGSolver::Solve( const floatspacearray & rhs,
				   const floatspacearray & init  ) {
    floatspacearray x,grad,cgrad,resid;
    double alfa,beta,determ;
    double cgnorm,csnorm,cgxr,csxr,cgxcs;

    floatspacearray step =  init.empty();
    floatspacearray cstep = rhs.empty();

    rhs_norm = rhs.norm();
    x = init; resid = rhs - op->Forward(x);

    numiter = 0;
    do{
	grad = op->Adjoint(resid);
	cgrad = op->Forward(grad);
	if (numiter==0) {
	    alfa = cgrad.dot(resid) / cgrad.norm2();
	    beta = 0;
	} else {
	    cgnorm = cgrad.norm2(); csnorm = cstep.norm2();
	    cgxcs = cgrad.dot(cstep);
	    cgxr = cgrad.dot(resid); csxr = cstep.dot(resid);
	    determ = cgnorm * csnorm - cgxcs * cgxcs + 1.0E-15;
	    alfa = (csnorm * cgxr - cgxcs * csxr) / determ;
	    beta = (-cgxcs * cgxr + cgnorm * csxr) / determ;
	}
	step = alfa * grad + beta * step;
	cstep = alfa * cgrad + beta * cstep;
	x += step; resid -= cstep;

  	// gather statistics, report, save iterations etc.
        resid_norm = resid.norm();
        report(); save_one_iter( x ); save_one_resid( resid );

    }while( (numiter++ < maxiter) && ( resid_norm/rhs_norm > 1e-8 ) );

    return x;
}
