
#ifndef LSSOLVER_H
#define LSSOLVER_H

// abstract base class for least squares solver, 
// the solve operation takes a RHS and an initial guess.
// and solves the problem min ( || op( x ) - rhs ||^2 + || x - x0 ||^2 )

#include <foperator.h>
#include <floatspacea.h>

class LSSolver {

public:

    LSSolver( foperator& op );

    void AddRightPrecond( foperator & m1 );
    void AddLeftPrecond( foperator & m2 );

    floatspacearray Solve( const floatspacearray & rhs, 
				   const floatspacearray & initial );

    virtual ~LSSolver();

protected:

    virtual floatspacearray solve_op( const floatspacearray & rhs,
                                      const floatspacearray & initial )=0;

    foperator * op;
    foperator * right_prec;
    foperator * left_prec;

};

#endif
